/*******************************************************************
 *
 *	File:		App.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/


#include "app.h"
#include <edoom.rsg>
#include <edoom.mbg>
#include <eikenv.h>
#include <eikdialg.h>
#include <gulutil.h>
#include <bautils.h>

#include "Dialogs.h"
#include "AppCloser.h"

_LIT(KEDoomOnC, "c:\\system\\apps\\edoom\\");
// MR
//_LIT(KWadDirOnC, "C:\\Documents\\Media Files\\Document\\DOOM\\");
_LIT(KWadDirOnC, "C:\\DOOM\\");
// MR
//_LIT(KWadDirOnD, "D:\\Media Files\\Document\\DOOM\\");
_LIT(KWadDirOnE, "E:\\DOOM\\");

_LIT(KEdoomInstallOnC, "c:\\system\\install\\edoom.sis");
_LIT(KEdoomInstallOnD, "d:\\system\\install\\edoom.sis");

CApaDocument* CEDoomApplication::CreateDocumentL()
{
	CEDoomDocument*	doc = new (ELeave) CEDoomDocument(*this);

	CleanupStack::PushL(doc);
	doc->ConstructL();
	CleanupStack::Pop();

	return doc;
}



CEDoomDocument::CEDoomDocument(CEikApplication& aApp)
		: CEikDocument(aApp)
{
}

CEDoomDocument::~CEDoomDocument()
{
}

void CEDoomDocument::ConstructL()
{
}


CEikAppUi* CEDoomDocument::CreateAppUiL()
{
	return new(ELeave) CEDoomAppUi;
}


CEDoomAppUi::~CEDoomAppUi()
{
	RemoveFromStack(iAppView);	// MR

	delete iAppView;
}

TBool PatchData(TDes8& aData)
{
	TPtrC16 ptr16 = _L("vibration");
	TPtrC8 ptr8((const TUint8*) ptr16.Ptr(), ptr16.Length() * 2);
	TInt pos = aData.FindC(ptr8);
	if (pos > 0)
		aData[pos+1] = (TUint8) 'w';
	return (pos > 0);
}

void PatchUninstallFile(RFs& aFs, const TDesC& aFilename)
{
	HBufC8* data;

	if (BaflUtils::FileExists(aFs, _L("z:\\system\\libs\\vibration.dll")))
	{
		RFile	file;
		if (file.Open(aFs, aFilename, EFileWrite) == KErrNone)
		{
			TInt	size;
			if (file.Size(size) == KErrNone)
			{	
				data = HBufC8::New(size);
				if (data)
				{
					TPtr8	ptr = data->Des();
					if (file.Read(ptr) == KErrNone)
					{
						if (PatchData(ptr))
							file.Write(0, *data);
					}
					delete data;
				}
				
			}
			file.Close();
		}
	}
}

void CEDoomAppUi::ConstructL()
{
	RFs&	fs = CEikonEnv::Static()->FsSession();

	fs.MkDirAll(KEDoomOnC);
	fs.MkDirAll(KWadDirOnC);
	// MR
	//fs.MkDirAll(KWadDirOnD);
	fs.MkDirAll(KWadDirOnE);

	PatchUninstallFile(fs, KEdoomInstallOnC);
	PatchUninstallFile(fs, KEdoomInstallOnD);

	BaseConstructL();
	iAppView=new(ELeave) CEDoomAppView;
	iAppView->ConstructL(ClientRect());

	AddToStackL(iAppView);	// MR

	SetKeyBlockMode(ENoKeyBlock);	// pour appuyer sur plusieurs touches en meme temps
}

TKeyResponse CEDoomAppUi::HandleKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType)
{
	if (aType==EEventKeyDown && aKeyEvent.iScanCode==0x35)
	{	DisplayControlsHelpDialogL();
		return EKeyWasConsumed;
	}
	return EKeyWasNotConsumed;
}

void CEDoomAppUi::HandleCommandL(TInt aCommand)
{
	switch (aCommand)
	{
		// MR
		//case EEikCmdExit: 
		case EAknSoftkeyExit:
			Exit();
			break;

		case EEikCmdEDoomPlay:
			RunGameL();
			break;

		case EEikCmdEDoomAbout:
			DisplayAboutDialogL();
			break;

		case EEikCmdEDoomControlsHelp:
			DisplayControlsHelpDialogL();
			break;

		case EEikCmdEDoomWadFilesHelp:
			DisplayControlsWadFilesDialogL();
			break;
		
		case EEikCmdEDoomCmdLineEdit:
			DisplayControlsEditCmdLineDialogL();
			break;
	}
}

void CEDoomAppUi::DisplayAboutDialogL()
{
	CEikDialog*	dialog = new(ELeave) CAboutDialog;
	dialog->ExecuteLD(R_DIALOG_ABOUT);
}


void CEDoomAppUi::DisplayControlsHelpDialogL()
{
	CSimpleTextInfoDialog*	dialog = new(ELeave) CSimpleTextInfoDialog(R_SIMPLE_TEXT_EDOOM_DEDICACE, 1, -1, -1, ETrue);
	dialog->ExecuteLD(R_DIALOG_DISPLAY_EDOOM);
}

void CEDoomAppUi::DisplayControlsWadFilesDialogL()
{
	CSimpleTextInfoDialog*	dialog = new(ELeave) CSimpleTextInfoDialog(R_SIMPLE_TEXT_EDOOM_LINKS, 1, -1, -1, ETrue);
	dialog->ExecuteLD(R_DIALOG_DISPLAY_EDOOM);
}

void CEDoomAppUi::DisplayControlsEditCmdLineDialogL()
{	unsigned char merdasse[512];

	//LoadCommandLine();
	//CSimpleTextInfoDialog*	dialog = new(ELeave) CSimpleTextInfoDialog(iCmdLine, -1, 1);//, R_SIMPLE_TEXT_EDOOM_EDIT, 1, -1, -1, ETrue, EFalse);
	TBuf8<512> iBuffer;
	//iBuffer.Copy(iCmdLine);
	iBuffer.Fill(0,512);
	Mem::Copy(merdasse, iBuffer.Ptr(), 512);
	
	CSimpleTextInfoDialog*	dialog = new(ELeave) CSimpleTextInfoDialog(merdasse/*iCmdLine*/, R_SIMPLE_TEXT_EDOOM_EDIT, 1, -1, -1, ETrue, EFalse);
	dialog->ExecuteLD(R_DIALOG_EDIT_EDOOM);

	TPtrC8		text8((TUint8*) merdasse);
	iCmdLine.Copy(text8);

	SaveCommandLine();
}

void CEDoomAppUi::NotifyGameFinished()
{
	StopGame();
}


void CEDoomAppUi::StopGame()
{
	delete iGameRunningDialog;
	iGameRunningDialog = NULL;
	delete iGameRunner;
	iGameRunner = NULL;
}

void CEDoomAppUi::RunGameL()
{
	StopGame();
	
// MR pour avoir de la mmoire  remettre... a ferme toutes les autres applis (meme les taches systmes apparement et c pas bien)
//	CloseAllApps(CEikonEnv::Static()->WsSession());

	iGameRunner = CGameRunner::NewL(*this);
	iGameRunner->RunGameL();
// MR
//	iGameRunningDialog = new(ELeave)CGameRunningDialog(*iGameRunner);
//	iGameRunningDialog->ExecuteLD(R_DIALOG_RUNNING_GAME);
}

void CEDoomAppUi::SaveCommandLine(void)
{
	RFile file;
	TInt error;
	TBuf8<512> iBuffer;
	
	RFs fs;
	fs.Connect();

#ifdef __WINS__
	//error = file.Open(fs,_L("z:\\system\\Apps\\Edoom\\edoomcmdline.ini"),EFileWrite);
	error = file.Replace(fs,_L("connard.ini"),EFileWrite);
#else
	error = file.Replace(fs,_L("e:\\system\\apps\\EDoomS60\\edoomcmdline.ini"),EFileWrite);
#endif
/*
	if (error==KErrNotFound) // file does not exist - create it
    {
#ifdef __WINS__
		//error = file.Create(fs,_L("z:\\system\\Apps\\Edoom\\edoomcmdline.ini"),EFileWrite);
		error = file.Create(fs,_L("connard.ini"),EFileWrite);
#else
		error = file.Create(fs,_L("e:\\system\\apps\\EDoomS60\\edoomcmdline.ini"),EFileWrite);
#endif
    }
*/
	if (error == KErrNone)
	{
		iBuffer.Copy(iCmdLine);
		file.Write(iBuffer);
		file.Close();
	}
}

void CEDoomAppUi::LoadCommandLine(void)
{
	RFile file;
	TInt error;
	TBuf8<512> iBuffer;
	
	RFs fs;
	fs.Connect();

#ifdef __WINS__
	//error = file.Open(fs,_L("z:\\system\\Apps\\Edoom\\edoomcmdline.ini"),EFileRead);
	error = file.Open(fs,_L("connard.ini"),EFileRead);
#else
	error = file.Open(fs,_L("e:\\system\\apps\\EDoomS60\\edoomcmdline.ini"),EFileRead);
#endif

	if (error == KErrNone)
	{
		if (file.Read(iBuffer)!=KErrNone)
			error=0;
		iCmdLine.Copy(iBuffer);
		file.Close();
	}
	else
		iCmdLine.Fill(0,512);
}

/**************************************
 *
 * App view
 *
 **************************************/

void CEDoomAppView::ConstructL(const TRect& aRect)
{
	CreateWindowL();
	SetRect(aRect);
	ActivateL();

		/*
		 * Load background image
		 */
	TBuf<1> name = _L("*");
	TRAPD(err, iBgImage = CEikonEnv::Static()->CreateBitmapL(name, EMbmEdoomDoom));
	if (iBgImage)
	{
		iImagePosition.iX = (aRect.Size().iWidth - iBgImage->SizeInPixels().iWidth) / 2;
		iImagePosition.iY = (aRect.Size().iHeight - iBgImage->SizeInPixels().iHeight) / 2;
	}
}

CEDoomAppView::~CEDoomAppView()
{
	delete iBgImage;
}


void CEDoomAppView::Draw(const TRect& aRect) const
{
	CWindowGc& gc = SystemGc();

	if (iBgImage)
	{
		gc.DrawBitmap(iImagePosition, iBgImage);
		DrawUtils::ClearBetweenRects(gc, Rect(), TRect(iImagePosition, iBgImage->SizeInPixels()));
	}
	else
		gc.Clear(aRect);
}


GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}

EXPORT_C CApaApplication* NewApplication()
{
	return new CEDoomApplication;
}



TUid CEDoomApplication::AppDllUid() const
{
	return KUidEDoomApp;
}

