/*******************************************************************
 *
 *	File:		AppCloser.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2003, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <e32base.h>
#include <w32std.h>
#include <apgtask.h>
#include <apgwgnam.h>

#include "AppCloser.h"

//#define __APP_CLOSER_TEST__
#ifdef __APP_CLOSER_TEST__
#define DPRINT(x) x
#else
#define DPRINT(x)
#endif

static void doCloseAllApps(RWsSession& aWsSession, TBool aForceKill)
{
	TApaTaskList	apaTaskList(aWsSession);

	TInt count = aWsSession.NumWindowGroups(0);
	CArrayFixFlat<TInt>* winGroupIds = new(ELeave) CArrayFixFlat<TInt>(count);
	CleanupStack::PushL(winGroupIds);

	aWsSession.WindowGroupList(0, winGroupIds);
	count = winGroupIds->Count();
	for (TInt i=0; i<count; i++)
	{
		TInt	wgId = (*winGroupIds)[i];
		TApaTask apaTask(aWsSession);
		apaTask.SetWgId(wgId);

		CApaWindowGroupName* wgName = CApaWindowGroupName::NewLC(aWsSession, wgId);
		TPtrC	caption = wgName->Caption();

			// ignore system tasks
		if (wgName->IsSystem())
		{
			DPRINT((RDebug::Print(_L("CLOSER: ignoring system task:%S"), &caption)));
			continue;
		}

			// ignore anything to do with EikonServer
		if (caption.Left(6).CompareF(_L("EIKSRV")) == KErrNone)
		{
			DPRINT((RDebug::Print(_L("CLOSER: ignoring EIKSRV task:%S"), &caption)));
			continue;
		}

			// ignore ourselves
		if (apaTask.ThreadId() == RThread().Id())
		{
			DPRINT((RDebug::Print(_L("CLOSER: ignoring myself:%S"), &caption)));
			continue;
		}

			// ignore tasks without caption
		if (caption.Length() == 0)
		{
			DPRINT((RDebug::Print(_L("CLOSER: ignoring captionless task"))));
			continue;
		}

		if (aForceKill)
		{
			DPRINT((RDebug::Print(_L("CLOSER: killing %S"), &caption)));
			apaTask.KillTask();
		}
		else
		{
			DPRINT((RDebug::Print(_L("CLOSER: shutting down  %S"), &caption)));
			apaTask.SendSystemEvent(EApaSystemEventShutdown);
		}
	}
	CleanupStack::PopAndDestroy(count);
	CleanupStack::PopAndDestroy(winGroupIds);
}


void CloseAllApps(RWsSession& aWsSession)
{
	DPRINT((RDebug::Print(_L("**********"))));

	User::InfoPrint(_L("Please wait, freeing memory..."));
	doCloseAllApps(aWsSession, EFalse);

	User::After(2 * 1000 * 1000);
	DPRINT((RDebug::Print(_L("**********"))));
	doCloseAllApps(aWsSession, ETrue);

	User::After(1 * 1000 * 1000);

#ifdef __APP_CLOSER_TEST__
		// just to see what is still hanging around
	User::After(2 * 1000 * 1000);
	RDebug::Print(_L("**********"));
	doCloseAllApps(aWsSession, ETrue);
#endif
}

