/*******************************************************************
 *
 *	File:		Dialogs.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "Dialogs.h"
#include "Engine.h"
#include "EDoom.hrh"
#include "EDoom.rsg"

#include "version.h"
#include "CSimpleTextParser.h"
#include <txtrich.h>				// CRichText
#include <eikrted.h>				// CEikRichTextEditor


/************************************************
 *
 * GameRunning Dialog
 *
 ************************************************/

CGameRunningDialog::CGameRunningDialog(CGameRunner& aGameRunner) : iGameRunner(aGameRunner)
{
}

void CGameRunningDialog::PreLayoutDynInitL()
{
}

void CGameRunningDialog::HandleControlStateChangeL(TInt aControlId)
{
#if 0	// MR
	switch (aControlId)
	{
		case ECtlCmdButPanelRunningGame:
		{
			//const TInt ctrlId=STATIC_CAST(CEikDialogToolBar*, Control(aControlId))->StateChangeControlId();
			const TInt ctrlId=((CEikDialogToolBar *)Control(aControlId))->StateChangeControlId();
			switch (ctrlId)
			{
				case ECtlCmdButStopGame:
					iGameRunner.TerminateGame();
					break;

				case ECtlCmdButResumeGame:
					iGameRunner.ResumeGame();
					break;
				default:
					break;
			}
			break;
		}
		default:
			break;
	}
#endif	// 0
}





/************************************************
 *
 * SimpleTextInfoDialog
 *
 ************************************************/


CSimpleTextInfoDialog::CSimpleTextInfoDialog(TInt aTextIdOne, TInt aRichTextCtlIdOne, TInt aTextIdTwo, TInt aRichTextCtlIdTwo, TBool aSimpleTextResIdOneIsArray, TBool aSimpleTextResIdTwoIsArray)
	: iRichTextCtlIdOne(aRichTextCtlIdOne), iSimpleTextResIdOne(aTextIdOne),
	  iRichTextCtlIdTwo(aRichTextCtlIdTwo), iSimpleTextResIdTwo(aTextIdTwo),
	  iSimpleTextResIdOneIsArray(aSimpleTextResIdOneIsArray), iSimpleTextResIdTwoIsArray(aSimpleTextResIdTwoIsArray),
	  iSetDialogBackground(ETrue), pcBuffer(NULL)
{
}

CSimpleTextInfoDialog::CSimpleTextInfoDialog(TUint8 *cBuffer/*TBuf<512> _iBuffer*/, TInt aTextIdOne, TInt aRichTextCtlIdOne, TInt aTextIdTwo, TInt aRichTextCtlIdTwo, TBool aSimpleTextResIdOneIsArray, TBool aSimpleTextResIdTwoIsArray)
	: iRichTextCtlIdOne(aRichTextCtlIdOne), iSimpleTextResIdOne(aTextIdOne),
	  iRichTextCtlIdTwo(aRichTextCtlIdTwo), iSimpleTextResIdTwo(aTextIdTwo),
	  iSimpleTextResIdOneIsArray(aSimpleTextResIdOneIsArray), iSimpleTextResIdTwoIsArray(aSimpleTextResIdTwoIsArray),
	  iSetDialogBackground(ETrue), pcBuffer(cBuffer)
{
}

void CSimpleTextInfoDialog::PreLayoutDynInitL()
{
	CEikRichTextEditor*		richTextEditor;
	
	if (iRichTextCtlIdOne != -1)
	{
		richTextEditor=STATIC_CAST(CEikRichTextEditor*, Control(iRichTextCtlIdOne));
		PreLayoutDynInitRichTextL(*richTextEditor, iRichTextCtlIdOne, iSimpleTextResIdOne);
	}

	if (iRichTextCtlIdTwo != -1)
	{
		richTextEditor=STATIC_CAST(CEikRichTextEditor*, Control(iRichTextCtlIdTwo));
		richTextEditor->Border().SetType(ENone);
		PreLayoutDynInitRichTextL(*richTextEditor, iRichTextCtlIdTwo, iSimpleTextResIdTwo);
	}
}


void CSimpleTextInfoDialog::PreLayoutDynInitRichTextL(CEikRichTextEditor& aRichTextEditor, TInt aRichTextCtlId, TInt aSimpleTextResId)
{
	iEikonEnv->BusyMsgL(_L("Opening"));
	aRichTextEditor.SetEdwinSizeObserver(this);
	if (iSetDialogBackground)
		aRichTextEditor.SetBackgroundColorL(iEikonEnv->Color(EColorDialogBackground));
	aRichTextEditor.SetSize(aRichTextEditor.Size()); // Set the size of the edwin

		// no scrollbars for us
	aRichTextEditor.CreateScrollBarFrameL(); // Create the scrollbars
	aRichTextEditor.ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff, iWantVertScrollbar ? CEikScrollBarFrame::EAuto: CEikScrollBarFrame::EOff);

	ShowTextL(*aRichTextEditor.RichText(), aRichTextCtlId, aSimpleTextResId);

	aRichTextEditor.UpdateAllFieldsL(); // Updates all the fields in the document

	aRichTextEditor.UpdateScrollBarsL();
}


void CSimpleTextInfoDialog::PostLayoutDynInitL()
{
	Layout();
	iEikonEnv->BusyMsgCancel();
}

TBool CSimpleTextInfoDialog::HandleEdwinSizeEventL(CEikEdwin* aEdwin, TEdwinSizeEvent aEventType, TSize aDesirableEdwinSize)
{
	if ((aEventType == EEventSizeChanging))
		aEdwin->SetSize(aDesirableEdwinSize);
	return EFalse;
}

TBool CSimpleTextInfoDialog::OkToExitL(TInt aButtonId)
{
#if 1
	if (pcBuffer)
	{
		TBuf8<512> iBuffer;
		//GetText(/*TDes& aDes*/iBuffer);
		TBuf<512>	iFilename;
		iFilename.Fill(0, 512);
		iBuffer.Fill(0, 512);
		GetEdwinText(iFilename, iRichTextCtlIdOne);
		iBuffer.Copy(iFilename);
		Mem::Copy(pcBuffer, iBuffer.Ptr(), 512);
	}
#else
	GetEdwinText(iBuffer, iRichTextCtlIdOne);
#endif
	return ETrue;
}

void CSimpleTextInfoDialog::ShowTextL(CRichText& aRichText, TInt /*aRichTextCtlId*/, TInt aResId)
{
	if (aResId != -1)
	{
		if ( ((aResId == iSimpleTextResIdOne) && (iSimpleTextResIdOneIsArray)) ||
		     ((aResId == iSimpleTextResIdTwo) && (iSimpleTextResIdTwoIsArray))
		   )
		{
			CDesCArrayFlat* desArray = CEikonEnv::Static()->ReadDesCArrayResourceL(aResId);
			CleanupStack::PushL(desArray);

			CSimpleTextFormatParser* parser = CSimpleTextFormatParser::NewLC();

			TInt	count = desArray->Count();
			for (TInt i=0 ;i<count ; i++)
				parser->ParseL(desArray->operator[](i), aRichText);

			CleanupStack::PopAndDestroy(parser);
			CleanupStack::PopAndDestroy(desArray);
		}
		else
		{
			HBufC*		text =	CEikonEnv::Static()->AllocReadResourceLC(aResId);
			ShowSimpleTextL(*text, aRichText);
			CleanupStack::PopAndDestroy(text);
		}
	}
	/*else
	{
		//HBufC*		text =	CEikonEnv::Static()->AllocReadResourceLC(aResId);
		ShowSimpleTextL(iBuffer, aRichText);
		//CleanupStack::PopAndDestroy(text);
	}*/
}

void CSimpleTextInfoDialog::ShowSimpleTextL(const TDesC& aSimpleText, CRichText& aRichText)
{
	CSimpleTextFormatParser* parser = CSimpleTextFormatParser::NewLC();
	parser->ParseL(aSimpleText, aRichText);

	CleanupStack::PopAndDestroy(parser);
}



/************************************************
 *
 * About Dialog
 *
 ************************************************/

//CAboutDialog::CAboutDialog() : CSimpleTextInfoDialog(-1, ECtlAboutVersion, R_SIMPLE_TEXT_ABOUT_LINKS, ECtlAboutLinks)
CAboutDialog::CAboutDialog() : CSimpleTextInfoDialog(R_SIMPLE_TEXT_ABOUT, ECtlAboutVersion, -1, ECtlAboutLinks)
{
}

void CAboutDialog::ShowTextL(CRichText& aRichText, TInt aRichTextCtlId, TInt aResId)
{
	if (aRichTextCtlId == ECtlAboutLinks)
		CSimpleTextInfoDialog::ShowTextL(aRichText, aRichTextCtlId, aResId);
	else
	{
		TBuf<16>	versionText;
		TBuf<512>	text;

#if (EMAME_VERSION_PATCH > 0)
		versionText.Format(_L("%d.%d.%d"), EDOOM_VERSION_MAJOR, EDOOM_VERSION_MINOR, EDOOM_VERSION_PATCH);
#else
		versionText.Format(_L("%d.%d"), EDOOM_VERSION_MAJOR, EDOOM_VERSION_MINOR);
#endif

		HBufC*		aboutFormat =	CEikonEnv::Static()->AllocReadResourceLC(R_SIMPLE_TEXT_ABOUT);
		text.Format(*aboutFormat, &versionText);

		ShowSimpleTextL(text, aRichText);

		CleanupStack::PopAndDestroy(aboutFormat);
	}
}
