/*******************************************************************
 *
 *	File:		EDoomMain.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <e32std.h>
#include <f32file.h>
#include <e32svr.h>
#include <estlib.h>
#include <eikdll.h>
#include <w32std.h>
#include <hal.h>

#include "game.h"


void DumpMemInfo()
{
	TInt	ramSize, ramSizeFree, romSize;
	
	HAL::Get(HALData::EMemoryRAM, ramSize);
	HAL::Get(HALData::EMemoryRAMFree, ramSizeFree);
	HAL::Get(HALData::EMemoryROM, romSize);

//	RDebug::Print(_L("ram=%dKB, ram_free=%dKB, rom=%dKB"), ramSize/1024, ramSizeFree/1024, romSize/1024);	
}




extern "C"
{
#include "doomdef.h"
#include "m_argv.h"
#include "d_main.h"

void D_DoomMainL() 
{
	D_DoomMain();
}

void D_DoomLoopL()
{
	D_DoomLoop();
}

ULONG I_GetTime();
}


int DoomL(int argc, char** argv) 
{ 
	myargc = argc; 
	myargv = argv; 

	DumpMemInfo();

	TRAPD(err, D_DoomMainL());
	if (!err)
	{
		TRAPD(err, D_DoomLoopL());
		if (err)
		{
//			RDebug::Print(_L("DoomL: D_DoomLoop error: err=%d"), err);
		}
	}
	else
	{
//		RDebug::Print(_L("DoomL: D_DoomMain error: err=%d"), err);
	}
	return err;
} 

void EDoomSleep(int aMs)
{
	User::After(aMs * 1000);
}


void EDoomDebugPrint(const char* format, ...)
{
	TBuf<128>	buf;
	char	text[128];
	va_list     argptr;

	va_start(argptr, format);
	vsprintf(text, format, argptr);
	va_end(argptr);

	TPtrC8	ptr((TText8*) text);
	buf.Copy(ptr);
	RDebug::Print(_L("%S"), &buf);
}


void EpocGameL(int argc, char* argv[])
{
#ifdef __WINS__
	int crt_argc=0;
	char** crt_argv=0;
	char** crt_envp=0;

	__crt0(crt_argc, crt_argv, crt_envp);
#endif
	
	printf(" ");
	fflush(stdout);	// MR

#ifdef __WINS__
	chdir("c:\\system\\apps\\edoom");	// MR
#else
	chdir("e:\\system\\apps\\edoom");	// MR
#endif

	DoomL(argc, argv);

#ifdef __WINS__
	RThread	me;
#else
	RProcess	me;
	me.Terminate(0);
#endif

#ifdef __WINS__
// MR
/*	fclose(stdin);
	fclose(stdout);
	fclose(stderr);*/
#endif
}







#ifdef __WINS1__

EXPORT_C void EntryRunGame(int argc, char* argv[])
{
	EpocGameL(argc, argv);
}


int __cdecl E32Dll(TDllReason aReason)
{
	return(KErrNone);
}

int main(int argc, char* argv[])
{
	DEBUGGER();
	return 0;
}



#else


void TargetEpocGameL(int argc, char* argv[])
{
	EpocGameL(argc, argv);
}


int TargetGame(int argc, char* argv[])
{
	CTrapCleanup* cleanup=CTrapCleanup::New();

	TRAPD(error, TargetEpocGameL(argc, argv));

	__ASSERT_ALWAYS(!error, User::Panic(_L("EDOOM"), error));

	delete cleanup;

	return 0;
}

//static char* argv[] = {"0000000000", "0000000000", "0000000000", "0000000000", "0000000000", "0000000000", "0000000000",
//					   "0000000000", "0000000000", "0000000000", "0000000000", "0000000000", "0000000000", "0000000000", "0000000000"};
static char **argv;

int strspndemerde(char *s1, char s2)
{ int i;

	for (i=0;i<strlen(s1);i++)
		if (s1[i]!=s2)
			return i;

	return strlen(s1);
}

void LoadCommandLine(int *argc/*, char *argv[]*/)
{
	RFile file;
	TInt error;
	TBuf8<512> iBuffer;
	char merdasse[512];
	
	RFs fs;
	fs.Connect();

#ifdef __WINS__
	//error = file.Open(fs,_L("z:\\system\\Apps\\Edoom\\edoomcmdline.ini"),EFileRead);
	error = file.Open(fs,_L("connard.ini"),EFileRead);
#else
	error = file.Open(fs,_L("e:\\system\\apps\\EDoomS60\\edoomcmdline.ini"),EFileRead);
#endif

	if (error == KErrNone)
	{
		iBuffer.Fill(0,512);
		if (file.Read(iBuffer)!=KErrNone)
			error=0;
		//iCmdLine.Copy(iBuffer);
		file.Close();
	}
	else
	{
		//iCmdLine.Fill(0,512);
		*argc = 0;
		return;
	}

	Mem::Copy(merdasse, iBuffer.Ptr(), 512);

	argv = new char *[15];
	for (int k=0;k<15;k++)
		argv[k] = new char[25];

	long j, i = 0;
	char *c;
	*argc = 1;	// 0 = name of the program...
	while (merdasse[i])
	{
		j = strspndemerde(merdasse+i, ' ')+i;
		if (j >= strlen(merdasse)-1)
			break;
		c = strstr(merdasse+j, " ");
		if (c)
		{	i = (long)c - (long)merdasse;
			strncpy(argv[*argc],merdasse+j, i-j);
			argv[*argc][i-j]=0;
			(*argc)++;
		}
		else
		{	strcpy(argv[*argc], merdasse+j);
			(*argc)++;
			break;
		}
		
		//(*argc)++;
		/*if (*argc==3)
			break;*/
	}
}


#ifdef __WINS__
EXPORT_C TInt WinsMain()
#else
GLDEF_C TInt E32Main()
#endif
{
	int argc;
	LoadCommandLine(&argc);

	TargetGame(argc, argv);

	return 0;
}


#if 0
extern "C" {
int main(int argc, char* argv[])
{
	TargetGame(argc, argv);

	return 0;
}
}		// extern "C"
#endif

#endif




#ifdef __WINS__
int __cdecl E32Dll(TDllReason aReason)
{
	return(KErrNone);
}
#endif