/*******************************************************************
 *
 *	File:		I_Video.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <stdlib.h>
#include <sys\unistd.h>
#include "video.h"


extern "C"
{
#include "doomdef.h"
#include "i_system.h"
#include "m_argv.h"
#include "v_video.h"
#include "st_stuff.h"
#include "i_video.h"
#include "keys.h"
#include "g_input.h"
#include "r_main.h"		// for R_ExecuteSetViewSize

// profiler support
#include "ll_video.h"
#include "profiler.h"

extern void D_PostEvent (const event_t* ev);

// synchronize page flipping with screen refresh
consvar_t       cv_vidwait = {"vid_wait","1",CV_SAVE,CV_OnOff};

/*********************************************************
 *
 * Global variables we are supposed to declare
 *
 *********************************************************/

// this is the CURRENT rendermode!! very important : used by w_wad, and many other code
rendermode_t    rendermode = render_soft;
boolean         highcolor = false;
byte			graphics_started = 0;

void I_UpdateNoBlit (void){}	// not needed
void I_StartFrame (void){}		// not needed

void I_ShutdownGraphics(void);
void I_FinishUpdate(void);
int VID_SetMode (int modenum);
void I_StartupGraphics(void);
void I_SetPalette (RGBA_t* palette);
void I_ReadScreen (byte* scr);
char *VID_GetModeName (int modenum);
int VID_NumModes(void);
int VID_GetModeForSize( int w, int h);


int IsPenDown();		// needed in g_game.c 
}		// extern "C"




/*********************************************************
 *
 * Global variables (for both video & keyboard)
 *
 *********************************************************/

static TInt sProfiler = 0;
void ToggleProfiler()
{
	if (sProfiler == 0)
	{
		profiler_start();
		sProfiler = 1;
	} 
	else
	{
		profiler_stop();
		sProfiler = 0;
	}
}


/*********************************************************
 *
 * Toolbar
 *
 *********************************************************/

#ifdef __WINS__
_LIT(KDoomMbmFile_C, "z:\\system\\Programs\\doom.mbm");
_LIT(KDoomMbmFile_D, "c:\\system\\Programs\\doom.mbm");
#else
_LIT(KDoomMbmFile_C, "c:\\system\\apps\\edoom\\doom.mbm");
_LIT(KDoomMbmFile_D, "d:\\system\\apps\\edoom\\doom.mbm");
#endif

const TInt KMaxVirtualButtons	= 15;
const TInt KButtonMargin		= 4;
const TInt KButtonFixedHeight	= 8;

const TInt KVirtualKeyboardHotkey = 240;

class CToolbar : public CBase, public MScreenOrientationObserver, public MGameScreenDrawObserver
{
public:
	static CToolbar* NewL(RWsSession& aWsSession, CGameScreenDevice* aGameScreenDevice);
// MR	static TVirtualButton	iVirtualButtons[KMaxVirtualButtons+1];

public:			// implements MScreenOrientationObserver
	virtual void ScreenOrientationChanged(TOrientation aOrientation);

public:			// implements MGameScreenDrawObserver
	virtual TBool	WantDrawToScreen() {return EFalse;}
	virtual void	PreDrawFrameToScreen(CGameScreenDevice* /*aDstGameScreenDevice*/){};
	virtual void	PostDrawFrameToScreen(CGameScreenDevice* aDstGameScreenDevice);

protected:
	CToolbar(RWsSession& aWsSession, CGameScreenDevice* aGameScreenDevice);
	~CToolbar();
	void ConstructL();
	void DrawToolBarL();
	void DrawButtonL(TInt aButton);

	CFbsGameScreenDevice*	iBitmap;
	RWsSession&				iWsSession;
	CGameScreenDevice*		iLastGameScreenDevice;
	CGameScreenDevice*		iGameScreenDevice;
	TBool					iIsRotLeft;

	TSize					iSize;
	TPoint					iPoint;

	TInt					iButtonWidth[KMaxVirtualButtons];
};

// MR
/*
TVirtualButton	CToolbar::iVirtualButtons[KMaxVirtualButtons+1] =
{
	{0, 0, 0, 0, EStdKeyEnter,	EKeyEnter},
	{0, 0, 0, 0, EStdKeyEscape, EKeyEscape},
	{0, 0, 0, 0, KVirtualKeyboardHotkey, KVirtualKeyboardHotkey},
	{0, 0, 0, 0, 1,				'1'},
	{0, 0, 0, 0, 2,				'2'},
	{0, 0, 0, 0, 3,				'3'},
	{0, 0, 0, 0, 4,				'4'},
	{0, 0, 0, 0, 5,				'5'},
	{0, 0, 0, 0, 6,				'6'},
	{0, 0, 0, 0, 7,				'7'},
	{0, 0, 0, 0, EStdKeyTab,	EKeyTab},
	{0, 0, 0, 0, 'y',			'y'},
	{0, 0, 0, 0, 'n',			'n'},
	{0, 0, 0, 0, EStdKeyF2,		KEY_F2},
	{0, 0, 0, 0, EStdKeyF3,		KEY_F3},
	{-1, -1, -1, -1, -1, -1}
};
*/
CToolbar::CToolbar(RWsSession& aWsSession, CGameScreenDevice* aGameScreenDevice) 
	: iWsSession(aWsSession), iGameScreenDevice(aGameScreenDevice)
{
}

CToolbar::~CToolbar()
{
	delete iBitmap;
}

CToolbar* CToolbar::NewL(RWsSession& aWsSession, CGameScreenDevice* aGameScreenDevice)
{
	CToolbar*	self = new(ELeave)CToolbar(aWsSession, aGameScreenDevice);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}


void CToolbar::ConstructL()
{
	// MR
#if 0
	iBitmap = CFbsGameScreenDevice::NewL(TSize(320, 8), iGameScreenDevice->DisplayMode());
#else
	iBitmap = CFbsGameScreenDevice::NewL(TSize(208, 8), iGameScreenDevice->DisplayMode());
#endif
	iBitmap->Clear();
	
// MR
	/*iSize.iHeight = KButtonFixedHeight;
	DrawToolBarL();*/
}

void CToolbar::ScreenOrientationChanged(TOrientation aOrientation)
{
	iIsRotLeft = (aOrientation == EOrientationRotLeft);
	iLastGameScreenDevice = NULL;	// forces  a redraw of the toolbar

	if (iIsRotLeft)
	{
#if 0	// MR
		iPoint = TPoint(207, 0);
	
		iVirtualButtons[0].iTlX = 200;
		iVirtualButtons[0].iTlY = 0;
		iVirtualButtons[0].iBrX = 208;
		iVirtualButtons[0].iBrY = iButtonWidth[0];
		for (TInt i=1 ; i<KMaxVirtualButtons ; i++)
		{
			iVirtualButtons[i].iTlX = 200;
			iVirtualButtons[i].iTlY = iVirtualButtons[i-1].iBrY + KButtonMargin;
			iVirtualButtons[i].iBrX = 208;
			iVirtualButtons[i].iBrY = iVirtualButtons[i].iTlY + iButtonWidth[i];
		}
#else
		iPoint = TPoint(175, 0);
#endif
	}
	else
	{
// MR
#if 0
		iVirtualButtons[0].iBrX = 8;
		iVirtualButtons[0].iBrY = 320;
		iVirtualButtons[0].iTlX = 0;
		iVirtualButtons[0].iTlY = 320  - iButtonWidth[0];
		for (TInt i=1 ; i<KMaxVirtualButtons ; i++)
		{
			iVirtualButtons[i].iBrX = 8;
			iVirtualButtons[i].iBrY = iVirtualButtons[i-1].iTlY - KButtonMargin;
			iVirtualButtons[i].iTlX = 0;
			iVirtualButtons[i].iTlY = iVirtualButtons[i].iBrY - iButtonWidth[i];
		}

		iPoint = TPoint(0, 319);
#else
		iPoint = TPoint(0, 207);
#endif
	}
}

void CToolbar::DrawButtonL(TInt aButton)
{
	CWsBitmap*	bitmap = new(ELeave)CWsBitmap(iWsSession);
	CleanupStack::PushL(bitmap);

	TInt err = bitmap->Load(KDoomMbmFile_C, aButton);
	if (err)
		err = bitmap->Load(KDoomMbmFile_D, aButton);
	User::LeaveIfError(err);
	TSize size = bitmap->SizeInPixels();
	if (size.iHeight != 8)
		User::Leave(KErrTooBig);

	CFbsBitGc*	gc = iBitmap->FbsBitGc();
	gc->BitBlt(TPoint(iSize.iWidth + KButtonMargin, 0), bitmap);
	iSize.iWidth += KButtonMargin + size.iWidth;
	iButtonWidth[aButton] = size.iWidth;

	CleanupStack::PopAndDestroy(bitmap);
}

void CToolbar::DrawToolBarL()
{
	for (TInt i=0 ; i<KMaxVirtualButtons ; i++)
		DrawButtonL(i);
}

void CToolbar::PostDrawFrameToScreen(CGameScreenDevice* aDstGameScreenDevice)
{
	if ((iLastGameScreenDevice != aDstGameScreenDevice) || (aDstGameScreenDevice!=iGameScreenDevice))
	{
		if (iIsRotLeft)
			aDstGameScreenDevice->DirectDraw_16bpp_16bppRotLeft(iPoint, *iBitmap, TPoint(0, 0), iSize);
		else
			aDstGameScreenDevice->DirectDraw_16bpp_16bppRotRight(iPoint, *iBitmap, TPoint(0, 0), iSize);
		iLastGameScreenDevice = aDstGameScreenDevice;
	}

};


/*********************************************************
 *
 * Framedrawer
 *
 *********************************************************/
typedef long fixed;			// Our new fixed point type.

#define itofx(x) ((x) << 8)			// Integer to fixed point
#define ftofx(x) ((x) * 256)			// Float to fixed point
#define dtofx(x) ((x) * 256)			// Double to fixed point
#define fxtoi(x) ((x) >> 8)			// Fixed point to integer
#define fxtof(x) ((float) (x) / 256)		// Fixed point to float
#define fxtod(x) ((double)(x) / 256)	        // Fixed point to double
#define Mulfx(x,y) (((y) * (x)) >> 8)		// Multiply a fixed by a fixed
#define Divfx(x,y) (((x) << 8) / (y))    		// Divide a fixed by a fixed

class TDoomFrameDrawer : public MGameScreenDrawer, public MScreenOrientationObserver, public MGameControlRawInputHandler, public MGameControlInputHandler
{
public:		// implements MGameScreenDrawer
	virtual void	DrawFrame(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice);
	virtual TRect	GameVisibleArea() const;
	virtual TSize	GameSize() const;
	virtual TPoint	GameOrigin() const;
	virtual void	SetGameOrigin(const TPoint& aOrigin);

public:		// implements MScreenOrientationObserver
	virtual void ScreenOrientationChanged(TOrientation aOrientation);

	virtual TBool ConsumeRawKeyEvent(TInt /*aType*/, const TKeyEvent& /*aKeyEvent*/);
	virtual TBool ConsumeRawPointerEvent(const TPointerEvent& /*aPointerEvent*/) { return ETrue; }
	virtual void HandleKeyEvent(TInt aType, const TKeyEvent& aKeyEvent);
	virtual void HandlePointerEvent(const TPointerEvent& /*aPointerEvent*/) {}

	void BlitImageRotLeft(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice);
	void BlitImageRotRight(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice);
	void BlitImageNoRot(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice);
	void PreComputeBilinearGrid(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice);

	//TBool		iIsRotLeft;
	TOrientation	aScreenOrientation;

	int iOffsetX;

	fixed *fraction_x, *fraction_y, *one_minus_x, *one_minus_y;
    int *ceil_x, *ceil_y, *floor_x, *floor_y;
	int *X,*Y;
};

TBool TDoomFrameDrawer::ConsumeRawKeyEvent(TInt /*aType*/, const TKeyEvent& aKeyEvent)
{
	if (aScreenOrientation==EOrientationNormal && (aKeyEvent.iModifiers & 0x500))	// Zoom mode & SHIFT key pressed
		return ETrue;
	else
		return EFalse;
}
void TDoomFrameDrawer::HandleKeyEvent(TInt aType, const TKeyEvent& aKeyEvent)
{
	static float speedX=1.0f;

	if (aType == EEventKey && aScreenOrientation==EOrientationNormal && (aKeyEvent.iModifiers & 0x500))
	{
		if (aKeyEvent.iScanCode == EStdKeyLeftArrow)
		{
			iOffsetX -= (int)speedX;	
			if (iOffsetX<=0)
				iOffsetX = 0;
			speedX*=2.0f;
		}
		if (aKeyEvent.iScanCode == EStdKeyRightArrow)
		{
			iOffsetX += (int)speedX;
			if (iOffsetX>=144)
				iOffsetX = 143;
			speedX*=2.0f;
		}
		if (aKeyEvent.iScanCode == EStdKeyDevice3)
			iOffsetX = 72;
	}
	else if (aType == EEventKeyUp)
	{
		speedX = 1.0f;
	}
}

void TDoomFrameDrawer::DrawFrame(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice)
{
	if (sProfiler)
		profiler_show();

	PROFILER_MARK(PROFILER_7);
	//if (iIsRotLeft)
	if (aScreenOrientation == EOrientationRotLeft)
		BlitImageRotLeft(aDstGameScreenDevice, aSrcGameScreenDevice);
	else
	if (aScreenOrientation == EOrientationRotRight)
		BlitImageRotRight(aDstGameScreenDevice, aSrcGameScreenDevice);
	else
		BlitImageNoRot(aDstGameScreenDevice, aSrcGameScreenDevice);

	PROFILER_MARK(PROFILER_END);
}

TRect TDoomFrameDrawer::GameVisibleArea() const
{
	return TRect(GameOrigin(), GameSize());
}

TSize TDoomFrameDrawer::GameSize() const
{
// MR
#if 0

#ifdef __WINS__
	return TSize(208, 320);
#else
	return TSize(200, 320);
#endif

#else	// 0
	return TSize(176, 208);
#endif	// 0
}

TPoint	TDoomFrameDrawer::GameOrigin() const
{
//#ifdef __WINS__
#if 1
	return TPoint(0,0);
#else
	return iIsRotLeft ? TPoint(0,0) : TPoint(8,0);
#endif
}

void TDoomFrameDrawer::SetGameOrigin(const TPoint& /*aOrigin*/)
{
}

void TDoomFrameDrawer::PreComputeBilinearGrid(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice)
{
	TInt					widthS = aSrcGameScreenDevice->Size().iWidth;
	TInt					heightS = aSrcGameScreenDevice->Size().iHeight;
	TInt					widthD = aDstGameScreenDevice->Size().iWidth;
	TInt					heightD = aDstGameScreenDevice->Size().iHeight;

	iOffsetX=72;

	fixed nXFactor = Divfx(itofx(widthS),itofx(heightD));
    fixed nYFactor = Divfx(itofx(heightS),itofx(widthD));

	floor_x = (int *)malloc(heightD*sizeof(int));
	floor_y = (int *)malloc(widthD*sizeof(int));
	ceil_x = (int *)malloc(heightD*sizeof(int));
	ceil_y = (int *)malloc(widthD*sizeof(int));
	floor_x = (int *)malloc(heightD*sizeof(int));
	floor_y = (int *)malloc(widthD*sizeof(int));
	ceil_x = (int *)malloc(heightD*sizeof(int));
	ceil_y = (int *)malloc(widthD*sizeof(int));
	fraction_x = (fixed *)malloc(heightD*sizeof(fixed));
	fraction_y = (fixed *)malloc(widthD*sizeof(fixed));
	one_minus_x = (fixed *)malloc(heightD*sizeof(fixed));
	one_minus_y = (fixed *)malloc(widthD*sizeof(fixed));
	X = (int *)malloc(heightD*sizeof(int));
	Y = (int *)malloc(widthD*sizeof(int));

	for(int i=0 ; i<widthD ; i++)
	{
        floor_y[i] = (int)fxtoi(Mulfx(itofx(i), nYFactor));
        ceil_y[i] = floor_y[i] + 1;
        if (ceil_y[i] >= heightS) ceil_y[i] = floor_y[i];
        fraction_y[i] = Mulfx(itofx(i), nYFactor) - itofx(floor_y[i]);
        one_minus_y[i] = itofx(1) - fraction_y[i];

		if (fraction_y[i]>128)	// nonbilinear blit
			Y[i]= ceil_y[i];
		else
			Y[i]= floor_y[i];

		for(int j=0 ; j<heightD ; j++)
		{
			floor_x[j] = (int)fxtoi(Mulfx(itofx(j), nXFactor));
            ceil_x[j] = floor_x[j] + 1;
            if (ceil_x[j] >= widthS) ceil_x[j] = floor_x[j];
            fraction_x[j] = Mulfx(itofx(j), nXFactor) - itofx(floor_x[j]);
            one_minus_x[j] = itofx(1) - fraction_x[j];

			if (fraction_x[j]>128)	// nonbilinear blit
				X[j]= ceil_x[j];
			else
				X[j]= floor_x[j];
		}
	}
}

#if 0
void TDoomFrameDrawer::BlitImageRotLeft(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice)
{
//#if 0	// MR  virer
	TInt					width = aSrcGameScreenDevice->Size().iWidth;
	TInt					height = aSrcGameScreenDevice->Size().iHeight;

	register TUint16*		frameBufferLine = (TUint16*) TGameBitmapUtil::PixelAddress(aDstGameScreenDevice, TPoint(height-1, 0));
	register TUint8*		src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(0);
	TUint*					pal = aSrcGameScreenDevice->Palette()->PaletteBuffer();

	for(int i=0 ; i<height ; i++)
	{
		TUint16*	dst = frameBufferLine;
		for(int j=0 ; j<width ; j++)
		{
			*dst = (TUint16) pal[*src++];
			dst += aDstGameScreenDevice->BytesPerLine()>>1;
		}
		frameBufferLine--;
	}
//#endif
}
#else

#ifndef __WINS__
extern "C" void DEB_Print(int i, int j, int k, int l);
#endif	// __WINS__

void TDoomFrameDrawer::BlitImageRotLeft(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice)
{
#if 1
	TInt					widthS = aSrcGameScreenDevice->Size().iWidth;
	TInt					widthD = aDstGameScreenDevice->Size().iWidth;
	TInt					heightD = aDstGameScreenDevice->Size().iHeight;

	register TUint16*		frameBufferLine = (TUint16*) TGameBitmapUtil::PixelAddress(aDstGameScreenDevice, TPoint(widthD-1, 0));
	TUint8*					srcA = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(0);
	TUint*					pal = aSrcGameScreenDevice->Palette()->PaletteBuffer();

	register int *_X,*_Y;
/*	static gni=0;
	if (DspMode==DSP_ALT)
	{
		if (gni)
		{
			_X=floor_x;
			_Y=floor_y;
		}
		else
		{
			_X=ceil_x;
			_Y=ceil_y;
		}
		gni=1-gni;
	}
	else*/
	{
		_X=X;
		_Y=Y;
	}

	TInt Inc = aDstGameScreenDevice->BytesPerLine()>>1;
	register TUint8* src=srcA;
	for(int i=0 ; i<widthD ; i++)
	{
		src = srcA + widthS * _Y[i];
		register TUint16*	dst = frameBufferLine;
		for(int j=0 ; j<heightD ; j++)
		{
			*dst = (TUint16) pal[src[_X[j]]];
			dst += Inc;
		}
		frameBufferLine--;
	}
#else
	TInt					widthS = aSrcGameScreenDevice->Size().iWidth;
	TInt					heightS = aSrcGameScreenDevice->Size().iHeight;
	TInt					widthD = aDstGameScreenDevice->Size().iWidth;
	TInt					heightD = aDstGameScreenDevice->Size().iHeight;
	float					srcCol;

	//DEB_Print(widthS, heightS, widthD, heightD);

	register TUint16*		frameBufferLine = (TUint16*) TGameBitmapUtil::PixelAddress(aDstGameScreenDevice, TPoint(widthD-1, 0));
	/*register*/ TUint8*		srcA = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(0);
	TUint*					pal = aSrcGameScreenDevice->Palette()->PaletteBuffer();

	register TUint8* src=srcA;
	for(int i=0 ; i<widthD ; i++)
	{
		srcCol = 0.0f;
		TUint16*	dst = frameBufferLine;
		for(int j=0 ; j<heightD ; j++)
		{
			//*dst = (TUint16) pal[*src++];
			*dst = (TUint16) pal[src[(TUint16)srcCol]];
			dst += aDstGameScreenDevice->BytesPerLine()>>1;
			//srcCol += (float)heightS/(float)widthD;
			srcCol += (float)widthS/(float)heightD;
			//srcCol += 1.3f;
		}
		src = srcA + widthS*(TUint8)((float)(i+1)*(float)heightS/(float)widthD);
		//src += widthS;
		frameBufferLine--;
	}
#endif
}

#endif

#if 0
void TDoomFrameDrawer::BlitImageRotRight(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice)
{
	TInt					width = aSrcGameScreenDevice->Size().iWidth;
	TInt					height = aSrcGameScreenDevice->Size().iHeight;

	register TUint16*		frameBufferLine = (TUint16*) TGameBitmapUtil::PixelAddress(aDstGameScreenDevice, TPoint(8, width-1));
	register TUint8*		src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(0);
	TUint*					pal = aSrcGameScreenDevice->Palette()->PaletteBuffer();

	for(int i=0 ; i<height ; i++)
	{
		TUint16*	dst = frameBufferLine;
		for(int j=0 ; j<width ; j++)
		{
			*dst = (TUint16) pal[*src++];
			dst -= aDstGameScreenDevice->BytesPerLine()>>1;
		}
		frameBufferLine++;
	}
}
#else	// 0
void TDoomFrameDrawer::BlitImageRotRight(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice)
{
#if 1
	TInt					widthS = aSrcGameScreenDevice->Size().iWidth;
	TInt					widthD = aDstGameScreenDevice->Size().iWidth;
	TInt					heightD = aDstGameScreenDevice->Size().iHeight;

	register TUint16*		frameBufferLine = (TUint16*) TGameBitmapUtil::PixelAddress(aDstGameScreenDevice, TPoint(0, heightD-1));
	TUint8*					srcA = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(0);
	TUint*					pal = aSrcGameScreenDevice->Palette()->PaletteBuffer();

	register int *_X,*_Y;
	/*static gni=0;
	if (DspMode==DSP_ALT)
	{
		if (gni)
		{
			_X=floor_x;
			_Y=floor_y;
		}
		else
		{
			_X=ceil_x;
			_Y=ceil_y;
		}
		gni=1-gni;
	}
	else*/
	{
		_X=X;
		_Y=Y;
	}

	TInt Inc = aDstGameScreenDevice->BytesPerLine()>>1;
	register TUint8* src=srcA;
	for(int i=0 ; i<widthD ; i++)
	{
		src = srcA + widthS * _Y[i];
		register TUint16*	dst = frameBufferLine;
		for(int j=0 ; j<heightD ; j++)
		{
			*dst = (TUint16) pal[src[_X[j]]];
			dst -= Inc;
		}
		frameBufferLine++;
	}
#else
	TInt					width = aSrcGameScreenDevice->Size().iWidth;
	TInt					height = aSrcGameScreenDevice->Size().iHeight;

	register TUint16*		frameBufferLine = (TUint16*) TGameBitmapUtil::PixelAddress(aDstGameScreenDevice, TPoint(8, width-1));
	register TUint8*		src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(0);
	TUint*					pal = aSrcGameScreenDevice->Palette()->PaletteBuffer();

	register TUint8* src=srcA;
	for(int i=0 ; i<widthD ; i++)
	{
		src = srcA + widthS * _Y[i];
		register TUint16*	dst = frameBufferLine;
		for(int j=0 ; j<heightD ; j++)
		{
			*dst = (TUint16) pal[src[_X[j]]];
			dst -= Inc;
		}
		frameBufferLine++;
	}
#endif
}
#endif	//0

void TDoomFrameDrawer::BlitImageNoRot(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice)
{
	TInt					widthS = aSrcGameScreenDevice->Size().iWidth;
	TInt					widthD = aDstGameScreenDevice->Size().iWidth;

	register TUint16*		frameBufferLine = (TUint16*) TGameBitmapUtil::PixelAddress(aDstGameScreenDevice, TPoint(0, 4));
	TUint8*					srcA = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(0);
	TUint*					pal = aSrcGameScreenDevice->Palette()->PaletteBuffer();

	TInt Inc = aDstGameScreenDevice->BytesPerLine()>>1;
	register TUint8* src=srcA;
	for(int j=0 ; j<200 ; j++)
	{
		src = srcA + iOffsetX + widthS * j;	// 72 = (320-176) / 2
		register TUint16*	dst = frameBufferLine;
		for(int i=0 ; i<widthD ; i++)
		{
			*dst = (TUint16) pal[src[i]];
			dst++;
		}
		frameBufferLine += Inc;
	}
}

void TDoomFrameDrawer::ScreenOrientationChanged(TOrientation aOrientation)
{
	//iIsRotLeft = aOrientation == EOrientationRotLeft;
	aScreenOrientation = aOrientation;
}



/****************************************************
 *
 * IMPLEMENTATION OF TDoomControlInputHandler
 *
 ****************************************************/

//const TInt KMaxLastDeltas = 3;
const TInt KDeferJogdialUpEventsDeltaFrames = 3;

class TDoomControlInputHandler : public MGameControlInputHandler, public MScreenOrientationObserver, public MGameScreenDrawObserver
{
public:
	TDoomControlInputHandler();
	TInt MapToKeyCode(TInt aScancode);
	void PostKeyEvent(TInt aKeyCode, TBool aUpEvent);

public:			// implements MScreenOrientationObserver
	virtual void ScreenOrientationChanged(TOrientation aOrientation);

public:		// implements MGameControlInputHandler
	virtual void HandleKeyEvent(TInt aType, const TKeyEvent& aKeyEvent);
	virtual void HandlePointerEvent(const TPointerEvent& /*aPointerEvent*/){};

public:		// implements MGameScreenDrawObserver
	virtual TBool	WantDrawToScreen() {return EFalse;}
	virtual void	PreDrawFrameToScreen(CGameScreenDevice* aDstGameScreenDevice);
	virtual void	PostDrawFrameToScreen(CGameScreenDevice* /*aDstGameScreenDevice*/){};

protected:
	static const TInt sKeyMapTableRotLeft[][2];
	static const TInt sKeyMapTableRotRight[][2];
	static const TInt sKeyMapTableNoRot[][2];
	
	//TBool		iIsRotLeft;
	TOrientation	aScreenOrientation;
	TInt		iJogdialDownInFrame[2];
	TInt		iFrames;

	TBool		isMovingForward;
};


const TInt TDoomControlInputHandler::sKeyMapTableRotLeft[][2] = 
{
	{GAME_LEFT_SCANCODE, KEY_LEFTARROW},
	{GAME_RIGHT_SCANCODE, KEY_RIGHTARROW},
	{GAME_UP_SCANCODE, KEY_UPARROW},
	{GAME_DOWN_SCANCODE, KEY_DOWNARROW},

#if 0	// MR
	{EStdKeyEscape, KEY_ESCAPE},
	{EStdKeyEnter, KEY_ENTER},

	{DEVICE_UP_SCANCODE, 'A'},
	{DEVICE_DOWN_SCANCODE, 'B'},
	{DEVICE_LEFT_SCANCODE, 'C'},
	{DEVICE_RIGHT_SCANCODE, 'D'},
	{DEVICE_SELECT_SCANCODE, 'E'},

	{EStdKeyApplication0, 'F'},
	{EStdKeyApplication1, 'G'},

	{EStdKeyTab, KEY_TAB},
#else	// 0
	{EStdKeyBackspace, KEY_ESCAPE},
	{EStdKeyDevice3, KEY_ENTER},	
	
	{DEVICE_UP_SCANCODE, KEY_LEFTARROW},
	{DEVICE_DOWN_SCANCODE, KEY_RIGHTARROW},
	{DEVICE_LEFT_SCANCODE, KEY_DOWNARROW},
	{DEVICE_RIGHT_SCANCODE, KEY_UPARROW},
	{DEVICE_SELECT_SCANCODE, KEY_ENTER},

	{EStdKeyApplication0, 'F'},
	{EStdKeyApplication1, 'G'},
	{EStdKeyDevice0, 'y'},

	{EStdKeyTab, KEY_TAB},
#endif	// 0
	{0, 0}
};

const TInt TDoomControlInputHandler::sKeyMapTableRotRight[][2] = 
{
#if 0
	{GAME_LEFT_SCANCODE, KEY_LEFTARROW},
	{GAME_RIGHT_SCANCODE, KEY_RIGHTARROW},
	{GAME_UP_SCANCODE, KEY_UPARROW},
	{GAME_DOWN_SCANCODE, KEY_DOWNARROW},

	{EStdKeyEscape, KEY_ESCAPE},
	{EStdKeyEnter, KEY_ENTER},

	{DEVICE_DOWN_SCANCODE, 'A'},
	{DEVICE_UP_SCANCODE, 'B'},
	{DEVICE_RIGHT_SCANCODE, 'C'},
	{DEVICE_LEFT_SCANCODE, 'D'},
	{DEVICE_SELECT_KEYCODE, 'E'},

	{EStdKeyApplication0, 'F'},
	{EStdKeyApplication1, 'G'},

	{EStdKeyTab, KEY_TAB},
#else	// 0
	{EStdKeyBackspace, KEY_ESCAPE},
	{EStdKeyDevice3, KEY_ENTER},	
	
	{DEVICE_UP_SCANCODE, KEY_RIGHTARROW},
	{DEVICE_DOWN_SCANCODE, KEY_LEFTARROW},
	{DEVICE_LEFT_SCANCODE, KEY_UPARROW},
	{DEVICE_RIGHT_SCANCODE, KEY_DOWNARROW},
	{DEVICE_SELECT_SCANCODE, KEY_ENTER},

	{EStdKeyApplication0, 'F'},
	{EStdKeyApplication1, 'G'},
	{EStdKeyDevice0, 'y'},

	{EStdKeyTab, KEY_TAB},
#endif	// 0
	{0, 0}
};

const TInt TDoomControlInputHandler::sKeyMapTableNoRot[][2] = 
{
	{GAME_LEFT_SCANCODE, KEY_LEFTARROW},
	{GAME_RIGHT_SCANCODE, KEY_RIGHTARROW},
	{GAME_UP_SCANCODE, KEY_UPARROW},
	{GAME_DOWN_SCANCODE, KEY_DOWNARROW},

	{EStdKeyBackspace, KEY_ESCAPE},
	{EStdKeyDevice3, KEY_ENTER},	
	
	{DEVICE_UP_SCANCODE, KEY_UPARROW},
	{DEVICE_DOWN_SCANCODE, KEY_DOWNARROW},
	{DEVICE_LEFT_SCANCODE, KEY_LEFTARROW},
	{DEVICE_RIGHT_SCANCODE, KEY_RIGHTARROW},
	{DEVICE_SELECT_SCANCODE, KEY_ENTER},

	{EStdKeyApplication0, 'F'},
	{EStdKeyApplication1, 'G'},
	{EStdKeyDevice0, 'y'},

	{EStdKeyTab, KEY_TAB},

	{0, 0}
};

TDoomControlInputHandler::TDoomControlInputHandler()
{
	iJogdialDownInFrame[0] = -1 - KDeferJogdialUpEventsDeltaFrames;
	iJogdialDownInFrame[1] = -1 - KDeferJogdialUpEventsDeltaFrames;
}

void TDoomControlInputHandler::ScreenOrientationChanged(TOrientation aOrientation)
{
	//iIsRotLeft = (aOrientation == EOrientationRotLeft);
	aScreenOrientation = aOrientation;
}

TInt TDoomControlInputHandler::MapToKeyCode(TInt aScancode)
{
	int i;

	//if (iIsRotLeft)
	if (aScreenOrientation == EOrientationRotLeft)
	{
		for (i = 0; sKeyMapTableRotLeft[i][0]; i++)
			if (sKeyMapTableRotLeft[i][0] == aScancode)
				return sKeyMapTableRotLeft[i][1];
	}
	else
	if (aScreenOrientation == EOrientationRotRight)
	{
		for (i = 0; sKeyMapTableRotRight[i][0]; i++)
			if (sKeyMapTableRotRight[i][0] == aScancode)
				return sKeyMapTableRotRight[i][1];
	}
	else
	{
		for (i = 0; sKeyMapTableNoRot[i][0]; i++)
			if (sKeyMapTableNoRot[i][0] == aScancode)
				return sKeyMapTableNoRot[i][1];
	}
	return aScancode;
}


void TDoomControlInputHandler::HandleKeyEvent(TInt aType, const TKeyEvent& aKeyEvent)
{
	if (aType == EEventKeyDown)
	{	// MR
		/*if (aKeyEvent.iScanCode == DEVICE_UP_SCANCODE)
		{
			iJogdialDownInFrame[0] = iFrames;
		}
		else if (aKeyEvent.iScanCode == DEVICE_DOWN_SCANCODE)
		{
			iJogdialDownInFrame[1] = iFrames;
		}*/
		PostKeyEvent(MapToKeyCode(aKeyEvent.iScanCode), EFalse);
	}
	else if (aType == EEventKeyUp)
	{
		// MR
		//if ((aKeyEvent.iScanCode != DEVICE_UP_SCANCODE) && (aKeyEvent.iScanCode != DEVICE_DOWN_SCANCODE))
			PostKeyEvent(MapToKeyCode(aKeyEvent.iScanCode), ETrue);
	}
	else if (aType == EEventKey)
	{
		if (isMovingForward && ( (aKeyEvent.iCode==1) || (aKeyEvent.iCode==2) ))
		{
			event_t ev;
			ev.type = ev_joystick;
			ev.data1 = 0;
			ev.data3 = 0;
			ev.data2 = (aKeyEvent.iCode==1) ? 75 : -75;
	//		RDebug::Print(_L("ev_mouse: aX=%d, aY=%d, mouseX=%d, mouseY=%d"), aX, aY, ev.data2, ev.data3);
			D_PostEvent(&ev);
		}
		if (aKeyEvent.iCode == 'p')
		{
			ToggleProfiler();
		}
	}
}

void TDoomControlInputHandler::PostKeyEvent(TInt aKeyCode, TBool aUpEvent)
{
	if (aKeyCode)
	{
		event_t ev;
		ev.type = aUpEvent ? ev_keyup : ev_keydown;
		ev.data1 = aKeyCode;
		ev.data2 = 0;
		ev.data3 = 0;
//		RDebug::Print(_L("D_PostEvent --> key=%i, up=%i"), aKeyCode, (TInt) aUpEvent);
		D_PostEvent(&ev);
	}
}


void TDoomControlInputHandler::PreDrawFrameToScreen(CGameScreenDevice* /*aDstGameScreenDevice*/)
{
	iFrames++;
// MR
/*	if (iJogdialDownInFrame[0] + KDeferJogdialUpEventsDeltaFrames == iFrames)
		PostKeyEvent(MapToKeyCode(DEVICE_UP_SCANCODE), ETrue);
	if (iJogdialDownInFrame[1] + KDeferJogdialUpEventsDeltaFrames == iFrames)
		PostKeyEvent(MapToKeyCode(DEVICE_DOWN_SCANCODE), ETrue);*/
}


class CSmartPenToMouseInputHandler : public CSmartPenInputHandler
{
public:
	static CSmartPenToMouseInputHandler* NewL(CGameWindow& aGameWindow, CGameController& aGameController);

protected:
	virtual void HandleMoveEvent(TInt aX, TInt aY);
	virtual void HandleUpEvent(){};
	virtual void HandleDownEvent(){};

protected:
	CSmartPenToMouseInputHandler(CGameWindow& aGameWindow, CGameController& aGameController);
	void ConstructL();
};


CSmartPenToMouseInputHandler::CSmartPenToMouseInputHandler(CGameWindow& aGameWindow, CGameController& aGameController)
	: CSmartPenInputHandler(aGameWindow, aGameController)
{
//	iAutoX = EFalse;
}

void CSmartPenToMouseInputHandler::ConstructL()
{
	CSmartPenInputHandler::ConstructL();
}


CSmartPenToMouseInputHandler* CSmartPenToMouseInputHandler::NewL(CGameWindow& aGameWindow, CGameController& aGameController)
{
	CSmartPenToMouseInputHandler*		self = new(ELeave)CSmartPenToMouseInputHandler(aGameWindow, aGameController);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}


void CSmartPenToMouseInputHandler::HandleMoveEvent(TInt aX, TInt aY)
{
	static lastY=0;
	static count = 0;
	if (++count != 3)
		return;
	count = 0;
	TPoint	delta = iLastPenPosition - iDownPenPosition;
	delta = TransformDeltaForOrientation(delta);
	if (abs(delta.iX) < 10)
		delta.iX = 0;
	if (abs(delta.iY) < 10)
		delta.iY = 0;

	aX = delta.iX;
	aY = delta.iY;

	if (aY != 0)
	{
		iDownPenPosition = iLastPenPosition;
		lastY = aY;
	}
	else
	{
		aY = lastY;
	}

	event_t ev;
	ev.type = ev_mouse;
	ev.data1 = 0;
	ev.data3 = (aY) ? ((aY > 0) ?  -250 : 250) : 0;
	if (iEventCount < 3)
		ev.data2 = (aX) ? ((aX > 0) ?  75 : -75) : 0;
	else
		ev.data2 = (aX) ? ((aX > 0) ?  150 : -150) : 0;
//	RDebug::Print(_L("ev_mouse: aX=%d, aY=%d, mouseX=%d, mouseY=%d"), aX, aY, ev.data2, ev.data3);
	D_PostEvent(&ev);
}


#define L_ROTATION_FILE_NAME "c:\\system\\apps\\edoom\\rotleft.ini"
#define R_ROTATION_FILE_NAME "c:\\system\\apps\\edoom\\rotright.ini"
void SaveRotationSetting(int mode/*TBool aIsLeft*/)
{
	FILE* file;

	switch(mode)
	{
		case 0:
			unlink(R_ROTATION_FILE_NAME);

			file=fopen(L_ROTATION_FILE_NAME, "w+");
			if (file)
			{
				fprintf(file, "\n");
				fclose(file);
			}
			break;
		case 1:
			unlink(L_ROTATION_FILE_NAME);

			file=fopen(R_ROTATION_FILE_NAME, "w+");
			if (file)
			{
				fprintf(file, "\n");
				fclose(file);
			}
			break;
		default:
			unlink(L_ROTATION_FILE_NAME);
			unlink(R_ROTATION_FILE_NAME);
	}
/*	if (aIsLeft)
	{
		unlink(R_ROTATION_FILE_NAME);
	}
	else
	{
		FILE* file=fopen(ROTATION_FILE_NAME, "w+");
		if (file)
		{
			fprintf(file, "\n");
			fclose(file);
		}
	}*/
}

/****************************************************
 *
 * IMPLEMENTATION OF VIDEO ROUTINES
 *
 ****************************************************/

static CGameWindow*				sGameWindow = NULL;
CMemGameScreenDevice*			sMemGameScreenDevice = NULL;
CGamePalette*					sCGamePalette = NULL;
TDoomFrameDrawer				sDoomFrameDrawer;
TDoomControlInputHandler		sDoomControlInputHandler;
CSmartPenToMouseInputHandler*	sSmartPenToMouseInputHandler;
CSmartPenToKeyInputHandler*		sSmartPenToKeyInputHandler;


#define MAXWINMODES 3
static int windowedModes[MAXWINMODES][2] = 
{
// MR
#if 1
	{320, 200},
	{320, 200},
	{320, 200}
#else
	{208,176},
	{208,176}
#endif
};

static char* windowedModesText[MAXWINMODES] = 
{
// MR
#if 0
	"320x200 L",
	"320x200 R",
	"176x200"
#else
	"208x176 L",
	"208x176 R",
	"176x200"
#endif
};



int VID_NumModes(void)
{
    return MAXWINMODES;
}

void I_StartupGraphics(void)
{
	if( graphics_started )
		return;

		// honor left and right rotation
	// MR
	if (access(R_ROTATION_FILE_NAME, 0) == 0)
		VID_SetMode(1);
	else
	if (access(L_ROTATION_FILE_NAME, 0) == 0)
		VID_SetMode(0);
	else
		VID_SetMode(2);

	graphics_started = true;
}


void I_ReadScreen (byte* scr)
{
	memcpy(scr, vid.buffer, vid.width*vid.height*vid.bpp);
}

void I_ShutdownGraphics(void)
{
    printf( "I_ShutdownGraphics\n");

    if( !graphics_started )
        return;

    graphics_started = false;
}

char *VID_GetModeName (int modenum)
{
   return windowedModesText[modenum];
}


int VID_GetModeForSize( int w, int h)
{
    int i;

    for (i=0; i<MAXWINMODES;i++)
        if(windowedModes[i][0]==w && windowedModes[i][1]==h)
            return i;
    return 0;
}


void I_FinishUpdate(void)
{
	static TInt sCount = 0;
	PROFILER_MARK(PROFILER_6);
	if (sCount++ == 3)
	{
		sCount = 0;
		return;
	}
	sGameWindow->GameFrameTick(sMemGameScreenDevice);
	PROFILER_MARK(PROFILER_END);
}



extern "C" 
{
	int oldMode = -1;
}
int VID_SetMode(int modenum)
{
	if (modenum >= MAXWINMODES) 
	{
		printf("VID_SetMode modenum >= MAXWINMODES\n");
		return -1;
	}
	if (oldMode == modenum)
		return 1;

	
		/*
		 * First time
		 */
	//TBool	isRotLeft = (modenum == 0);
	if (oldMode == -1)
	{
		TSize	gameSize(windowedModes[modenum][0], windowedModes[modenum][1]);
		sGameWindow = CGameWindow::NewL(_L("EDOOM"));
		sGameWindow->SetGameScreenDrawer(&sDoomFrameDrawer);
		sGameWindow->AddScreenOrientationObserverL(sDoomFrameDrawer);
		sGameWindow->GameController().AddGameControlInputHandlerL(sDoomFrameDrawer);	// MR ZOOM
		sGameWindow->GameController().AddGameControlRawInputHandlerL(sDoomFrameDrawer);	// MR ZOOM
		sGameWindow->GameController().AddGameControlInputHandlerL(sDoomControlInputHandler);
		sGameWindow->AddGameScreenDrawObserverL(sDoomControlInputHandler);
		sGameWindow->AddScreenOrientationObserverL(sDoomControlInputHandler);

		// MR
		/*CVirtualKeyboard&		virtualKeyboard = sGameWindow->VirtualKeyboard();
		virtualKeyboard.SetKeyCodesAreScanCodes(ETrue);
		virtualKeyboard.SetPosition(TPoint(0, 50));
		*/
		int vidbuffersize = (gameSize.iWidth * gameSize.iHeight * NUMSCREENS)
			+ (gameSize.iWidth * ST_HEIGHT);  //status bar

		vid.width  = gameSize.iWidth;
		vid.height = gameSize.iHeight;
//		printf("VID_SetMode %dx%d\n", vid.width, vid.height);

		vid.direct = vid.buffer = (unsigned char*) malloc(vidbuffersize);
		vid.rowbytes = vid.width;
		vid.bpp      = 1;
		vid.recalc   = 1;

		sMemGameScreenDevice = CMemGameScreenDevice::NewL(TSize(vid.width, vid.height), EColor256, (TUint8*) vid.direct);

		sDoomFrameDrawer.PreComputeBilinearGrid(sGameWindow->GameScreenDevice(), sMemGameScreenDevice);
// MR
/*
//		sSmartPenToMouseInputHandler = CSmartPenToMouseInputHandler::NewL(*sGameWindow, sGameWindow->GameController());
		sSmartPenToKeyInputHandler = CSmartPenToKeyInputHandler::NewL(*sGameWindow, sGameWindow->GameController());
		sSmartPenToKeyInputHandler->AutoForward(ETrue);

		CToolbar*	toolbar = CToolbar::NewL(sGameWindow->WsSession(), sGameWindow->GameScreenDevice());
		sGameWindow->AddGameScreenDrawObserverL(*toolbar);
		sGameWindow->AddScreenOrientationObserverL(*toolbar);
// MR
/*		virtualKeyboard.SetVirtualButtons(toolbar->iVirtualButtons);
		virtualKeyboard.SetHotKeyForVirtualKeyboard(KVirtualKeyboardHotkey);*/
	}
	else
	{
		sGameWindow->ClearScreen();
		SaveRotationSetting(modenum/*isRotLeft*/);
	}

	oldMode = modenum;
	//sGameWindow->SetScreenOrientation(isRotLeft ? EOrientationRotLeft : EOrientationRotRight);
	switch(modenum)
	{
		case 0:
			sGameWindow->SetScreenOrientation(EOrientationRotLeft);
			break;
		case 1:
			sGameWindow->SetScreenOrientation(EOrientationRotRight);
			break;
		default:
			sGameWindow->SetScreenOrientation(EOrientationNormal);
	}
	vid.modenum  = modenum;

	printf("VID_SetMode(%d) DONE\n", modenum);
	return 1;
}


void I_SetPalette (RGBA_t* palette)
{
	ASSERT(sMemGameScreenDevice);


	TDisplayMode	displayMode = sGameWindow->GameScreenDevice()->DisplayMode();
	if (!sCGamePalette)
		sCGamePalette = CGamePalette::NewL(displayMode);

	byte*  usegamma;
	usegamma = gammatable[cv_usegamma.value%5];

	for(int i=0 ; i<256 ; i++)
	{
/*		int r = usegamma[*palette++];
		int g = usegamma[*palette++];
		int b = usegamma[*palette++];
*/
		int r = usegamma[palette->s.red];
		int g = usegamma[palette->s.green];
		int b = usegamma[palette->s.blue];
		palette++;
		TRgb	color(r, g, b);
		(sCGamePalette->PaletteBuffer())[i] = (TUint16) TGameBitmapUtil::GetColorValue(color, displayMode);
	}
	sMemGameScreenDevice->SetPalette(sCGamePalette);


#ifdef __USE_PROFILER
	(sCGamePalette->PaletteBuffer())[255] = (TUint16) TGameBitmapUtil::GetColorValue(KRgbWhite, displayMode);
	video_setup((unsigned*) vid.direct, vid.width, vid.height, 8, 255, 0);
	ToggleProfiler();
#endif
}

int osd_cycles()
{
	return User::TickCount();
}



/*
 * needed in g_game.c 
 */
int IsPenDown()
{
	if (sSmartPenToKeyInputHandler)
		return sSmartPenToKeyInputHandler->IsPenDown();
	return 0;
}
