#include "stdio.h"
#include "profiler.h"
#include "ll_video.h"

#ifdef __WINS__
typedef __int64 UINT64;
#else
typedef long long UINT64;
#endif


void ui_text(const char* text, int line)
{
	video_puts_pos(text, 0, line);
}



static int use_profiler;
static int s_prev_lines=0;
#define MEMORY 6

struct profile_data
{
	UINT64 count[MEMORY][PROFILER_TOTAL];
};

static struct profile_data profile;
static int memory;


static int FILO_type[10];
static unsigned int FILO_start[10];
static int FILO_length;

void profiler_start(void)
{
	use_profiler = 1;
	FILO_length = 0;
}

void profiler_stop(void)
{
	use_profiler = 0;
}

void profiler_mark(int type)
{
#if 0	// MR
	unsigned int curr_cycles;


	if (!use_profiler)
	{
		FILO_length = 0;
		return;
	}

	curr_cycles = osd_cycles();

	if (type != PROFILER_END)
	{
		if (FILO_length >= 10)
		{
//logerror("Profiler error: FILO buffer overflow\n");
			return;
		}

		if (FILO_length > 0)
		{
			/* handle nested calls */
			profile.count[memory][FILO_type[FILO_length-1]] += (unsigned int)(curr_cycles - FILO_start[FILO_length-1]);
		}
		FILO_type[FILO_length] = type;
		FILO_start[FILO_length] = curr_cycles;
		FILO_length++;
	}
	else
	{
		if (FILO_length <= 0)
		{
//logerror("Profiler error: FILO buffer underflow\n");
			return;
		}

		profile.count[memory][FILO_type[FILO_length-1]] += (unsigned int)(curr_cycles - FILO_start[FILO_length-1]);
		FILO_length--;
		if (FILO_length > 0)
		{
			/* handle nested calls */
			FILO_start[FILO_length-1] = curr_cycles;
		}
	}
#endif	// MR
}

void profiler_show(void)
{
#if 0	// MR
	int i,j;
	UINT64 total,normalize;
	UINT64 computed;
	int line;
	char buf[30];
	static char *names[PROFILER_TOTAL] =
	{
		"Prof1",
		"Prof2",
		"Prof3",
		"Prof4",
		"Prof5",
		"Prof6",
		"Prof7",
		"Prof8",
		"Prof9",
		"ProfA",
		"Profilr"
	};


	if (!use_profiler) return;

	profiler_mark(PROFILER_PROFILER);

	computed = 0;
	i = 0;
	while (i < PROFILER_PROFILER)
	{
		for (j = 0;j < MEMORY;j++)
			computed += profile.count[j][i];
		i++;
	}
	normalize = computed;
	while (i < PROFILER_TOTAL)
	{
		for (j = 0;j < MEMORY;j++)
			computed += profile.count[j][i];
		i++;
	}
	total = computed;

	if (total == 0 || normalize == 0) return;	/* we have been just reset */

	line = 0;
	if (s_prev_lines)
		video_clear_area(0, 0, 20, s_prev_lines);
	for (i = 0;i < PROFILER_TOTAL;i++)
	{
		computed = 0;
		{
			for (j = 0;j < MEMORY;j++)
				computed += profile.count[j][i];
		}
//		if (computed)
		{
			if (i < PROFILER_PROFILER)
				sprintf(buf,"%s%3d%%%3d%%",names[i],
						(int)((computed * 100 + total/2) / total),
						(int)((computed * 100 + normalize/2) / normalize));
			else
				sprintf(buf,"%s%3d%%",names[i],
						(int)((computed * 100 + total/2) / total));
			ui_text(buf, line++);
		}
	}
	s_prev_lines = line;

	/* reset the counters */
	memory = (memory + 1) % MEMORY;
	for (i = 0;i < PROFILER_TOTAL;i++)
		profile.count[memory][i] = 0;

	profiler_mark(PROFILER_END);
#endif	// 0
}
